/**
 * Execute commands.
 * @memberof module:apeman/lib
 * @function cli
 * @returns {Promise}
 * @example
 *    cli(process.argv)
 */

'use strict'

const execcli = require('execcli')
const bin = require.resolve('../bin/apeman')
const path = require('path')

/** @lends cli */
function cli (argv) {
  let cwd = _cwd(argv[ 1 ])
  let args = [].concat(argv).slice(2)
  return new Promise((resolve, reject) => {
    execcli(bin, args, { cwd }, (err) =>
      err ? reject(err) : resolve(true)
    )
  })
}

function _cwd (filename) {
  try {
    return require(filename)[ '$cwd' ] || require.resolve(path.dirname(filename))
  } catch (e) {
    return null
  }
}

module.exports = cli
