/**
 * Create apeman app with process fork.
 * @function frk
 */
'use strict'

const argx = require('argx')
const co = require('co')
const childProcess = require('child_process')

/** @lends frk */
function frk (filename) {
  let working, restartTimer
  return {
    /**
     * Start the process.
     * @returns {Promise}
     */
    start () {
      const s = this
      if (working) {
        s.stop()
      }
      return new Promise((resolve, reject) => {
        process.nextTick(() => {
          working = childProcess.fork(filename)
          // noinspection Eslint
          s.__proto__ = working
          resolve()
        })
      })
    },
    /**
     * Stop the process.
     * @returns {Promise}
     */
    stop () {
      const s = this
      return new Promise((resolve, reject) => {
        setTimeout(() => {
          if (working) {
            working.send('apeman:frk:stop')
            working.once('close', () => {
              resolve()
            })
            working.kill()
            working = null
          }
          // noinspection Eslint
          s.__proto__ = {}
        }, 10) // DELAY to wait finish process like sse
      })
    },
    /**
     * Restart the process.
     * @returns {Promise}
     */
    restart () {
      if (working) {
        working.send('apeman:frk:restart')
      }

      const s = this
      return co(function * () {
        yield s.stop()
        yield s.start()
      })
    },
    /**
     * Restart the process with delay.
     * @param {number} delay - Delay to restart.
     * @returns {Promise}
     */
    restartWithDelay (delay) {
      let args = argx(arguments)
      delay = args.shift('number') || 300
      const s = this
      return co(function * () {
        clearTimeout(restartTimer)
        yield new Promise((resolve) => {
          restartTimer = setTimeout(() => {
            resolve()
          }, delay)
        })
        yield s.restart()
      })
    }
  }
}

module.exports = frk
